#!/usr/bin/env ruby

Signal.trap("PIPE", "EXIT")

require "abort_if"
require "fileutils"
require "parse_fasta"
require "optimist"
require "set"

include AbortIf
include AbortIf::Assert

opts = Optimist.options do
  banner <<-EOS

  This will pick genomes that have a host in specified phyla.  You can
  also specify a minimum genome length (say if you wanted to make it
  more simlar to the GOV contig set.

  The options are the same as for the pick_gov_contigs script even
  though that VHDB has genomes.

  Phylums will be matched ignoring case.

  Some records have multiple viral genome IDs.  These will be ignored.

  Columns of the input file: virushostdb.tsv from virus-host db ftp.

    0.  virus tax id
    1.  virus name
    2.  virus lineage
    3.  refseq id
    4.  KEGG GENOME
    5.  KEGG DISEASE
    6.  DISEASE
    7.  host tax id
    8.  host name
    9.  host lineage
    10. pmid
    11. evidence
    12. sample type
    13. source organism

  Options:
  EOS

  opt(:predicted_host_file,
      "File with host info",
      type: :string)
  opt(:contigs_file,
      "Viral contigs",
      type: :string)
  opt(:keep_these_phyla,
      "List of phyla to keep",
      type: :string)
  opt(:outdir,
      "Output directory",
      type: :string,
      default: ".")
  opt(:basename,
      "Basename for outfiles",
      default: "gov")

  opt(:min_contig_length,
      "Min. length of a contig to keep",
      default: 20000)
end

min_contig_length = opts[:min_contig_length]
basename = opts[:basename]

# Handle host_file arg
abort_unless opts[:predicted_host_file_given],
             "--host_file is a required argument."

host_file = opts[:predicted_host_file]
abort_unless File.exist?(host_file),
             "#{host_file} does not exist."

# Handle contigs_file arg
abort_unless opts[:contigs_file_given],
             "--contigs_file is a required argument."

contigs_file = opts[:contigs_file]
abort_unless File.exist?(contigs_file),
             "#{contigs_file} does not exist."

# Handle phyla arg
phyla_file = opts[:keep_these_phyla]
abort_unless opts[:keep_these_phyla_given],
             "--keep-these-phyla is a required arg."
abort_unless File.exist?(phyla_file),
             "#{phyla_file} does not exist."

# Handle outdir arg
outdir = opts[:outdir]
FileUtils.mkdir_p outdir

# These are the phyla we want to keep
keep_phyla = File.open(phyla_file, "rt").
             read.
             chomp.
             split(" ").
             map(&:downcase)

virus_info = Hash.new do |ht, id|
  ht[id] = {
    vtaxa: Set.new,
    htaxa: Set.new
  }
end

# First, read the virushostdb.tsv file
File.open(host_file, "rt").each_line.with_index do |line, idx|
  line.chomp!

  ary = line.split "\t"

  host_lineage = ary[9]
  virus_genome_id = ary[3]
  virus_lineage = ary[2]

  if idx.zero?
    abort_if host_lineage != "host lineage",
             "The 10th column should be 'host lineage', " \
             "got '#{host_lineage}'"
  elsif !host_lineage.nil? && !virus_genome_id.nil?
    host_lineage_split = host_lineage.split "; "

    # Sometimes there are multiple genome ids for a particular virus.
    # In these cases, we skip this record.
    virus_genome_id_split = virus_genome_id.split(", ")
    if virus_genome_id_split.count == 1
      virus_genome_id = virus_genome_id_split.first



      # Need at least domain and phylum.  Also the first should be
      # Bacteria and the second should be one of the ones mentioned
      # above.
      if host_lineage_split.count >= 2
        phylum = host_lineage_split[1].downcase

        # So it has a host, but does it have a *.viridae taxa
        # designation?
        vtaxa = nil
        virus_lineage.split("; ").each do |taxa|
          if taxa.match /viridae\Z/i
            # We pick the first viridae designation
            vtaxa = taxa.downcase
            break
          end
        end

        if keep_phyla.include?(phylum) && !vtaxa.nil?
          virus_info[virus_genome_id][:vtaxa] = vtaxa
          virus_info[virus_genome_id][:htaxa] = phylum
        end
      end
    end
  end
end

host_outfname = File.join outdir, "#{basename}.host_info.txt"
contigs_outfname = File.join outdir, "#{basename}.contigs.fa"

# Now go through the genomes and pick the sequences that we need.
File.open(host_outfname, "w") do |host_outf|
  File.open(contigs_outfname, "w") do |contigs_outf|
    ParseFasta::SeqFile.open(contigs_file).each_record do |rec|
      if rec.seq.length > min_contig_length &&
         virus_info.has_key?(rec.id)

        contigs_outf.puts rec
        host_outf.puts [rec.id,
                        virus_info[rec.id][:vtaxa],
                        virus_info[rec.id][:htaxa]].join "\t"

      end
    end
  end
end
